; A simple example of Direct3D 8 programming with MASM.

; How to compile
;
; ml /c /coff /Ic:\masm\include vector.asm
; ml /c /coff /Ic:\masm\include matrix.asm
; ml /c /coff /Ic:\masm\include d3dsetup.asm
; ml /c /coff /Ic:\masm\include d3d8_ex1.asm
; link /SUBSYSTEM:WINDOWS /MACHINE:IX86 /OPT:NOREF /ENTRY:Start /OUT:d3d8_ex1.exe d3d8_ex1.obj d3dsetup.obj matrix.obj vector.obj
;
; Change the path in the /I option to whatever folder matches your system.

; /Mic, 2003
; stabmaster_@hotmail.com
; http://www.cyd.liu.se/~micol972/site/


.486
.model flat,stdcall
option casemap:none


include .\d3d8.inc
include .\macros.inc
include .\matrix.inc
include .\d3dsetup.inc
include kernel32.inc
include user32.inc
include gdi32.inc

; You may have to change these if you keep MASM somewhere else.
includelib \masm\lib\kernel32.lib
includelib \masm\lib\user32.lib
includelib \masm\lib\gdi32.lib



; Window dimensions
XDIM	EQU 640
YDIM	EQU 480



; This is the format of the vertexes i'll use to define the triangle.
CUSTOM_VERTEX STRUCT 
	x	REAL4 ?
	y	REAL4 ?
	z	REAL4 ?
	col	dd ?
CUSTOM_VERTEX ENDS



.data
	; This is defined in d3dsetup.asm and will contain a pointer to a IDirect3DDevice8
	; interface after CreateD3DDevice has been called.
	EXTERN lpD3DDevice:DWORD

	winCaption	db 'DirectX 8 for MASM',0
	myClassName	db 'DX8MASMCLASS',0

	xtrans		REAL4 0.0
	ytrans		REAL4 0.0
	ztrans		REAL4 -1.5
	
	fov		REAL4 1.0472	; Field of vision: 60 * PI / 180
	aspect		REAL4 0.75	; Aspect ration: height/width
	zNear		REAL4 1.0	; Distance to near clipping plane
	zFar		REAL4 100.0	; Distance to far clipping plane

	; This is the triangle. The colors are in ARGB format.
	vertexes	CUSTOM_VERTEX < 0.0,  0.7, 0.0, 0ffff0000h>,
	                              <-0.7, -0.7, 0.0, 0ff00ff00h>,
	                              < 0.7, -0.7, 0.0, 0ff0000ffh>

		
.data?
	hWindow		dd ?
	
	viewMatrix	D3DXMATRIX <?>
	projMatrix	D3DXMATRIX <?>
	worldMatrix	D3DXMATRIX <?>
	
	
.code



; This function clears the background and draws a shaded triangle.
Draw PROC
	pusha
	
	; Cleat the target with color 00000000h (black).
	; COINVOKE is defined in macros.inc. It's used to call functions through a COM interface.
	COINVOKE lpD3DDevice,IDirect3DDevice8,Clear, 0, NULL, D3DCLEAR_TARGET, 00000000h, 0, 0

	; Beginning of scene.	
	COINVOKE lpD3DDevice,IDirect3DDevice8,BeginScene

		; Set the vertex format.
		COINVOKE lpD3DDevice,IDirect3DDevice8,SetVertexShader, D3DFVF_XYZ or D3DFVF_DIFFUSE

		; Draw a triangle.		
		COINVOKE lpD3DDevice,IDirect3DDevice8,DrawPrimitiveUP, D3DPT_TRIANGLELIST, 1, ADDR vertexes, sizeof CUSTOM_VERTEX

	; End of scene.
	COINVOKE lpD3DDevice,IDirect3DDevice8,EndScene

	; Swap the front and back buffers.
	COINVOKE lpD3DDevice,IDirect3DDevice8,Present, NULL, NULL, NULL, NULL

	popa
	ret
Draw ENDP




; This functions creates a Direct3D device and sets some initial parameters.
Init PROC
	pusha
	
	; Create a D3D device. The parameters are:
	;  * Handle of owner window
	;  * Width of owner window
	;  * Height of owner window
	;  * Desired bits per pixel
	;  * Fullscreen flag (true/false)
	;
	invoke CreateD3DDevice,hWindow,XDIM,YDIM,32,0
	.if eax == FALSE
		invoke ExitProcess,0
		popa
		ret
	.endif

	; Disable Z-buffering.
	COINVOKE lpD3DDevice,IDirect3DDevice8,SetRenderState, D3DRS_ZENABLE, D3DZB_FALSE
	
	; Disable lighting.
	COINVOKE lpD3DDevice,IDirect3DDevice8,SetRenderState, D3DRS_LIGHTING, FALSE
	
	; Set the culling mode to no culling.
	COINVOKE lpD3DDevice,IDirect3DDevice8,SetRenderState, D3DRS_CULLMODE, D3DCULL_NONE


	; Transform the world matrix to move all objects "backwards".
	invoke _D3DXMatrixTranslation, ADDR worldMatrix, xtrans,ytrans, ztrans
	COINVOKE lpD3DDevice,IDirect3DDevice8,SetTransform, D3DTS_WORLD, ADDR worldMatrix
	
	; Set up a projection matrix. I'm using a right-handed coordinate system here, i.e.
	; Z grows "towards the screen".
	invoke _D3DXMatrixPerspectiveFovRH, ADDR projMatrix, fov, aspect, zNear, zFar
	COINVOKE lpD3DDevice,IDirect3DDevice8,SetTransform, D3DTS_PROJECTION, ADDR projMatrix
		
	popa
	ret
Init ENDP




WndProc PROC hWnd:HWND,uMsg:UINT,wParam:WPARAM,lParam:LPARAM
	LOCAL crect:RECT
	LOCAL ps:PAINTSTRUCT

 	mov eax,uMsg

 	.if eax==WM_PAINT
    		invoke BeginPaint,hWnd,ADDR ps

		; Draw another frame    		
    		invoke Draw
    		
    		invoke EndPaint,hWnd,ADDR ps
    		xor eax,eax
  
  	.elseif eax==WM_KEYDOWN
    		.if wParam==VK_ESCAPE
      			invoke PostMessage,hWnd,WM_CLOSE,NULL,NULL
    		.endif
		xor eax,eax
		
  	.elseif eax==WM_DESTROY
    		invoke PostQuitMessage,0
 
	.else
 		invoke DefWindowProc,hWnd,uMsg,wParam,lParam
 	.endif
 
 	ret
WndProc ENDP




WinMain PROC hInst:DWORD,prevInstance:DWORD,cmdlinePtr:DWORD,cmdShow:DWORD
	LOCAL wc:WNDCLASSEX
	LOCAL msg:MSG

 	lea edi,wc
 	xor eax,eax
 	mov ecx,(sizeof WNDCLASSEX)/4
 	rep stosd
 	mov eax,hInst
  	mov wc.hInstance,eax
 	mov wc.cbSize, sizeof WNDCLASSEX
 	mov wc.style,CS_OWNDC or CS_HREDRAW or CS_VREDRAW
 	mov wc.lpszClassName,offset myClassName
 	mov wc.lpfnWndProc,offset WndProc
 	invoke LoadCursor,NULL,IDC_ARROW
 	mov wc.hCursor,eax
 	mov wc.hbrBackground,0

 	invoke RegisterClassEx,ADDR wc

 	sub eax,eax 
 	invoke CreateWindowEx,NULL,ADDR myClassName,ADDR winCaption,WS_OVERLAPPEDWINDOW OR WS_CLIPCHILDREN OR WS_CLIPSIBLINGS,80,80,XDIM,YDIM,eax,eax,hInst,eax
 	mov hWindow,eax

	; Initialize
	invoke Init

 	invoke ShowWindow,eax,SW_SHOWDEFAULT
 	invoke UpdateWindow,hWindow  

 	@@winmainmsgloop:
  		invoke InvalidateRect,hWindow,NULL,FALSE 
  		sub eax,eax
  		lea edi,msg
  		invoke GetMessage,edi,eax,eax,eax
  		test eax,eax
  		jz @@quitmsgposted
  		push edi
  		push edi
  		call TranslateMessage
  		call DispatchMessage
  	jmp @@winmainmsgloop
 
 	EVEN
 
 	@@quitmsgposted:

	; Destroy the D3D device.  
  	invoke DestroyD3DDevice
  	
  	mov eax,msg.wParam
 	Abort::
  	invoke ExitProcess,eax
  
  	ret
WinMain ENDP



Start:

invoke GetModuleHandle,NULL 
invoke WinMain,eax,NULL,NULL,NULL

END Start

